@@ -1,29 +1,26 @@
-/*****************************************************************************
- * VLC backend for the Phonon library                                        *
- * Copyright (C) 2007-2008 Tanguy Krotoff <tkrotoff@gmail.com>               *
- * Copyright (C) 2008 Lukas Durfina <lukas.durfina@gmail.com>                *
- * Copyright (C) 2009 Fathi Boudra <fabo@kde.org>                            *
- *                                                                           *
- * This program is free software; you can redistribute it and/or             *
- * modify it under the terms of the GNU Lesser General Public                *
- * License as published by the Free Software Foundation; either              *
- * version 2.1 of the License, or (at your option) any later version.        *
- *                                                                           *
- * This program is distributed in the hope that it will be useful,           *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
- * Lesser General Public License for more details.                           *
- *                                                                           *
- * You should have received a copy of the GNU Lesser General Public          *
- * License along with this package; if not, write to the Free Software       *
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA *
- *****************************************************************************/
+/*
+    Copyright (C) 2007-2008 Tanguy Krotoff <tkrotoff@gmail.com>
+    Copyright (C) 2008 Lukas Durfina <lukas.durfina@gmail.com>
+    Copyright (C) 2009 Fathi Boudra <fabo@kde.org>
+    Copyright (C) 2013 Harald Sitter <sitter@kde.org>
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "audiooutput.h"
 
-#ifdef PHONON_PULSESUPPORT
 #include <phonon/pulsesupport.h>
-#endif
 
 #include <vlc/vlc.h>
 
@@ -31,6 +28,7 @@
 #include "utils/debug.h"
 #include "devicemanager.h"
 #include "mediaobject.h"
+#include "media.h"
 
 namespace Phonon {
 namespace VLC {
@@ -45,19 +43,21 @@ AudioOutput::~AudioOutput()
 {
 }
 
-void AudioOutput::connectToMediaObject(MediaObject *mediaObject)
+void AudioOutput::handleConnectToMediaObject(MediaObject *mediaObject)
 {
-    SinkNode::connectToMediaObject(mediaObject);
+    Q_UNUSED(mediaObject);
     setOutputDeviceImplementation();
-#ifdef PHONON_PULSESUPPORT // Only mess with the volume if no PA is active!
     if (!PulseSupport::getInstance()->isActive())
-#endif
         applyVolume();
 }
 
-void AudioOutput::disconnectFromMediaObject(MediaObject *mediaObject)
+void AudioOutput::handleAddToMedia(Media *media)
 {
-    SinkNode::disconnectFromMediaObject(mediaObject);
+    media->addOption(":audio");
+    PulseSupport *pulse = PulseSupport::getInstance();
+    if (pulse && pulse->isActive()) {
+        pulse->setupStreamEnvironment(m_streamUuid);
+    }
 }
 
 qreal AudioOutput::volume() const
@@ -90,6 +90,7 @@ bool AudioOutput::setOutputDevice(int deviceIndex)
     return setOutputDevice(device);
 }
 
+#if (PHONON_VERSION >= PHONON_VERSION_CHECK(4, 2, 0))
 bool AudioOutput::setOutputDevice(const AudioOutputDevice &newDevice)
 {
     debug() << Q_FUNC_INFO;
@@ -109,24 +110,30 @@ bool AudioOutput::setOutputDevice(const AudioOutputDevice &newDevice)
 
     return true;
 }
+#endif
+
+#if (PHONON_VERSION >= PHONON_VERSION_CHECK(4, 6, 50))
+void AudioOutput::setStreamUuid(QString uuid)
+{
+    m_streamUuid = uuid;
+}
+#endif
 
 void AudioOutput::setOutputDeviceImplementation()
 {
     Q_ASSERT(m_player);
-#ifdef PHONON_PULSESUPPORT
     if (PulseSupport::getInstance()->isActive()) {
         m_player->setAudioOutput("pulse");
         debug() << "Setting aout to pulse";
         return;
     }
-#endif
 
     const QVariant dalProperty = m_device.property("deviceAccessList");
     if (!dalProperty.isValid()) {
         error() << "Device" << m_device.property("name") << "has no access list";
         return;
     }
-    const DeviceAccessList deviceAccessList = qVariantValue<DeviceAccessList>(dalProperty);
+    const DeviceAccessList deviceAccessList = dalProperty.value<DeviceAccessList>();
     if (deviceAccessList.isEmpty()) {
         error() << "Device" << m_device.property("name") << "has an empty access list";
         return;
