@@ -2,8 +2,8 @@
     Copyright (C) 2007-2008 Tanguy Krotoff <tkrotoff@gmail.com>
     Copyright (C) 2008 Lukas Durfina <lukas.durfina@gmail.com>
     Copyright (C) 2009 Fathi Boudra <fabo@kde.org>
-    Copyright (C) 2009-2011 vlc-phonon AUTHORS
-    Copyright (C) 2011 Harald Sitter <sitter@kde.org>
+    Copyright (C) 2009-2011 vlc-phonon AUTHORS <kde-multimedia@kde.org>
+    Copyright (C) 2011-2013 Harald Sitter <sitter@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -25,17 +25,16 @@
 #include <QtCore/QLatin1Literal>
 #include <QtCore/QtPlugin>
 #include <QtCore/QVariant>
-#include <QtGui/QMessageBox>
+#include <QMessageBox>
 
 #include <phonon/GlobalDescriptionContainer>
-#ifdef PHONON_PULSESUPPORT
 #include <phonon/pulsesupport.h>
-#endif
 
 #include <vlc/libvlc_version.h>
 
 #include "audio/audiooutput.h"
 #include "audio/audiodataoutput.h"
+#include "audio/volumefadereffect.h"
 #include "devicemanager.h"
 #include "effect.h"
 #include "effectmanager.h"
@@ -43,12 +42,18 @@
 #include "sinknode.h"
 #include "utils/debug.h"
 #include "utils/libvlc.h"
+#include "utils/mime.h"
 #ifdef PHONON_EXPERIMENTAL
 #include "video/videodataoutput.h"
 #endif
+#ifndef PHONON_NO_GRAPHICSVIEW
+#include "video/videographicsobject.h"
+#endif
 #include "video/videowidget.h"
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 Q_EXPORT_PLUGIN2(phonon_vlc, Phonon::VLC::Backend)
+#endif
 
 namespace Phonon
 {
@@ -81,18 +86,41 @@ Backend::Backend(QObject *parent, const QVariantList &)
     // Actual libVLC initialisation
     if (LibVLC::init()) {
         debug() << "Using VLC version" << libvlc_get_version();
-        QString userAgent =
-                QString("%0/%1 (Phonon/%2; Phonon-VLC/%3)").arg(
-                    QCoreApplication::applicationName(),
-                    QCoreApplication::applicationVersion(),
-                    PHONON_VERSION_STR,
-                    PHONON_VLC_VERSION);
-        libvlc_set_user_agent(libvlc,
-                              QCoreApplication::applicationName().toUtf8().constData(),
-                              userAgent.toUtf8().constData());
+        if (!qApp->applicationName().isEmpty()) {
+            QString userAgent =
+                    QString("%0/%1 (Phonon/%2; Phonon-VLC/%3)").arg(
+                        qApp->applicationName(),
+                        qApp->applicationVersion(),
+                        PHONON_VERSION_STR,
+                        PHONON_VLC_VERSION);
+            libvlc_set_user_agent(libvlc,
+                                  qApp->applicationName().toUtf8().constData(),
+                                  userAgent.toUtf8().constData());
+        } else {
+            qWarning("WARNING: Setting the user agent for streaming and"
+                     " PulseAudio requires you to set QCoreApplication::applicationName()");
+        }
+#ifdef __GNUC__
+#warning application name ought to be configurable by the consumer ... new api
+#endif
+#if (LIBVLC_VERSION_INT >= LIBVLC_VERSION(2, 1, 0, 0))
+        if (!qApp->applicationName().isEmpty() && !qApp->applicationVersion().isEmpty()) {
+            const QString id = QString("org.kde.phonon.%1").arg(qApp->applicationName());
+            const QString version = qApp->applicationVersion();
+            const QString icon = qApp->applicationName();
+            libvlc_set_app_id(libvlc,
+                              id.toUtf8().constData(),
+                              version.toUtf8().constData(),
+                              icon.toUtf8().constData());
+        } else if (PulseSupport::getInstance()->isActive()) {
+            qWarning("WARNING: Setting PulseAudio context information requires you"
+                     " to set QCoreApplication::applicationName() and"
+                     " QCoreApplication::applicationVersion()");
+        }
+#endif
     } else {
 #ifdef __GNUC__
-#warning TODO - this error message is about as useful as a cooling unit in the arctic
+    #warning TODO - this error message is as useful as a knife at a gun fight
 #endif
         QMessageBox msg;
         msg.setIcon(QMessageBox::Critical);
@@ -106,13 +134,11 @@ Backend::Backend(QObject *parent, const QVariantList &)
         fatal() << "Phonon::VLC::vlcInit: Failed to initialize VLC";
     }
 
-#ifdef PHONON_PULSESUPPORT
     // Initialise PulseAudio support
     PulseSupport *pulse = PulseSupport::getInstance();
     pulse->enable();
     connect(pulse, SIGNAL(objectDescriptionChanged(ObjectDescriptionType)),
             SIGNAL(objectDescriptionChanged(ObjectDescriptionType)));
-#endif
 
     m_deviceManager = new DeviceManager(this);
     m_effectManager = new EffectManager(this);
@@ -126,12 +152,10 @@ Backend::~Backend()
         delete GlobalAudioChannels::self;
     if (GlobalSubtitles::self)
         delete GlobalSubtitles::self;
-#ifdef PHONON_PULSESUPPORT
     PulseSupport::shutdown();
-#endif
 }
 
-QObject *Backend::createObject(BackendInterface::Class c, QObject *parent, const QList<QVariant> &/*args*/)
+QObject *Backend::createObject(BackendInterface::Class c, QObject *parent, const QList<QVariant> &args)
 {
     if (!LibVLC::self || !libvlc)
         return 0;
@@ -141,13 +165,9 @@ QObject *Backend::createObject(BackendInterface::Class c, QObject *parent, const
         return new MediaObject(parent);
     case AudioOutputClass:
         return new AudioOutput(parent);
-#ifdef __GNUC__
-#warning using sout in VLC2 breaks libvlcs vout functions, see vlc bug 6992
-// https://trac.videolan.org/vlc/ticket/6992
-#endif
 #if (LIBVLC_VERSION_INT < LIBVLC_VERSION(2, 0, 0, 0))
-    // FWIW: the case is inside the if because that gives clear indication which
-    // frontend objects are not supported!
+    // Broken >= 2.0
+    // https://trac.videolan.org/vlc/ticket/6992
     case AudioDataOutputClass:
         return new AudioDataOutput(parent);
 #endif
@@ -155,8 +175,19 @@ QObject *Backend::createObject(BackendInterface::Class c, QObject *parent, const
     case VideoDataOutputClass:
         return new VideoDataOutput(parent);
 #endif
+#ifndef PHONON_NO_GRAPHICSVIEW
+    case VideoGraphicsObjectClass:
+        return new VideoGraphicsObject(parent);
+#endif
+    case EffectClass:
+        return effectManager()->createEffect(args[0].toInt(), parent);
     case VideoWidgetClass:
         return new VideoWidget(qobject_cast<QWidget *>(parent));
+//    case VolumeFaderEffectClass:
+#ifdef __GNUC__
+#warning VFE crashes and has volume bugs ... deactivated
+//        return new VolumeFaderEffect(parent);
+#endif
     }
 
     warning() << "Backend class" << c << "is not supported by Phonon VLC :(";
@@ -165,98 +196,8 @@ QObject *Backend::createObject(BackendInterface::Class c, QObject *parent, const
 
 QStringList Backend::availableMimeTypes() const
 {
-    if (m_supportedMimeTypes.isEmpty()) {
-        const_cast<Backend *>(this)->m_supportedMimeTypes
-                << QLatin1String("application/ogg")
-                << QLatin1String("application/x-ogg")
-                << QLatin1String("application/vnd.rn-realmedia")
-                << QLatin1String("application/x-annodex")
-                << QLatin1String("application/x-flash-video")
-                << QLatin1String("application/x-quicktimeplayer")
-                << QLatin1String("application/x-extension-mp4")
-                << QLatin1String("audio/168sv")
-                << QLatin1String("audio/3gpp")
-                << QLatin1String("audio/3gpp2")
-                << QLatin1String("audio/8svx")
-                << QLatin1String("audio/aiff")
-                << QLatin1String("audio/amr")
-                << QLatin1String("audio/amr-wb")
-                << QLatin1String("audio/basic")
-                << QLatin1String("audio/mp3")
-                << QLatin1String("audio/mp4")
-                << QLatin1String("audio/midi")
-                << QLatin1String("audio/mpeg")
-                << QLatin1String("audio/mpeg2")
-                << QLatin1String("audio/mpeg3")
-                << QLatin1String("audio/prs.sid")
-                << QLatin1String("audio/vnd.rn-realaudio")
-                << QLatin1String("audio/vnd.rn-realmedia")
-                << QLatin1String("audio/wav")
-                << QLatin1String("audio/webm")
-                << QLatin1String("audio/x-16sv")
-                << QLatin1String("audio/x-8svx")
-                << QLatin1String("audio/x-aiff")
-                << QLatin1String("audio/x-basic")
-                << QLatin1String("audio/x-it")
-                << QLatin1String("audio/x-m4a")
-                << QLatin1String("audio/x-matroska")
-                << QLatin1String("audio/x-mod")
-                << QLatin1String("audio/x-mp3")
-                << QLatin1String("audio/x-mpeg")
-                << QLatin1String("audio/x-mpeg2")
-                << QLatin1String("audio/x-mpeg3")
-                << QLatin1String("audio/x-mpegurl")
-                << QLatin1String("audio/x-ms-wma")
-                << QLatin1String("audio/x-ogg")
-                << QLatin1String("audio/x-pn-aiff")
-                << QLatin1String("audio/x-pn-au")
-                << QLatin1String("audio/x-pn-realaudio-plugin")
-                << QLatin1String("audio/x-pn-wav")
-                << QLatin1String("audio/x-pn-windows-acm")
-                << QLatin1String("audio/x-real-audio")
-                << QLatin1String("audio/x-realaudio")
-                << QLatin1String("audio/x-s3m")
-                << QLatin1String("audio/x-speex+ogg")
-                << QLatin1String("audio/x-vorbis+ogg")
-                << QLatin1String("audio/x-wav")
-                << QLatin1String("audio/x-xm")
-                << QLatin1String("image/ilbm")
-                << QLatin1String("image/png")
-                << QLatin1String("image/x-ilbm")
-                << QLatin1String("image/x-png")
-                << QLatin1String("video/3gpp")
-                << QLatin1String("video/3gpp2")
-                << QLatin1String("video/anim")
-                << QLatin1String("video/avi")
-                << QLatin1String("video/divx")
-                << QLatin1String("video/flv")
-                << QLatin1String("video/mkv")
-                << QLatin1String("video/mng")
-                << QLatin1String("video/mp4")
-                << QLatin1String("video/mpeg")
-                << QLatin1String("video/mpeg-system")
-                << QLatin1String("video/mpg")
-                << QLatin1String("video/msvideo")
-                << QLatin1String("video/ogg")
-                << QLatin1String("video/quicktime")
-                << QLatin1String("video/webm")
-                << QLatin1String("video/x-anim")
-                << QLatin1String("video/x-flic")
-                << QLatin1String("video/x-flv")
-                << QLatin1String("video/x-matroska")
-                << QLatin1String("video/x-mng")
-                << QLatin1String("video/x-m4v")
-                << QLatin1String("video/x-mpeg")
-                << QLatin1String("video/x-mpeg-system")
-                << QLatin1String("video/x-ms-asf")
-                << QLatin1String("video/x-ms-wma")
-                << QLatin1String("video/x-ms-wmv")
-                << QLatin1String("video/x-ms-wvx")
-                << QLatin1String("video/x-msvideo")
-                << QLatin1String("video/x-quicktime")
-                << QLatin1String("audio/x-flac")
-                << QLatin1String("audio/x-ape");
-    }
+    if (m_supportedMimeTypes.isEmpty())
+        const_cast<Backend *>(this)->m_supportedMimeTypes = mimeTypeList();
     return m_supportedMimeTypes;
 }
 
@@ -276,7 +217,7 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
     }
     break;
     case Phonon::EffectType: {
-        QList<EffectInfo *> effectList = effectManager()->effects();
+        QList<EffectInfo> effectList = effectManager()->effects();
         for (int eff = 0; eff < effectList.size(); ++eff) {
             list.append(eff);
         }
@@ -310,12 +251,12 @@ QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescripti
     }
     break;
     case Phonon::EffectType: {
-        QList<EffectInfo *> effectList = effectManager()->effects();
+        const QList<EffectInfo> effectList = effectManager()->effects();
         if (index >= 0 && index <= effectList.size()) {
-            const EffectInfo *effect = effectList[ index ];
-            ret.insert("name", effect->name());
-            ret.insert("description", effect->description());
-            ret.insert("author", effect->author());
+            const EffectInfo &effect = effectList.at(index);
+            ret.insert("name", effect.name());
+            ret.insert("description", effect.description());
+            ret.insert("author", effect.author());
         } else {
             Q_ASSERT(1); // Since we use list position as ID, this should not happen
         }
@@ -359,13 +300,11 @@ bool Backend::connectNodes(QObject *source, QObject *sink)
             return true;
         }
 
-        /*
-        Effect *effect = qobject_cast<Effect *>(source);
+        VolumeFaderEffect *effect = qobject_cast<VolumeFaderEffect *>(source);
         if (effect) {
-            // FIXME connect the effect
+            sinkNode->connectToMediaObject(effect->mediaObject());
             return true;
         }
-        */
     }
 
     warning() << "Linking" << source->metaObject()->className() << "to" << sink->metaObject()->className() << "failed";
@@ -384,13 +323,11 @@ bool Backend::disconnectNodes(QObject *source, QObject *sink)
             return true;
         }
 
-        /*
-        Effect *effect = qobject_cast<Effect *>(source);
+        VolumeFaderEffect *const effect = qobject_cast<VolumeFaderEffect *>(source);
         if (effect) {
-            // FIXME disconnect the effect
+            sinkNode->disconnectFromMediaObject(effect->mediaObject());
             return true;
         }
-        */
     }
 
     return false;
